@@ -44,6 +44,7 @@ def new(settings):
     formatter_name = classname(settings.FORMAT, 'Formatter')
     if formatter_name not in globals():
         raise RuntimeError("Formatter not found: '%s'." % settings.FORMAT)
+    logger.debug("Creating new %s", formatter_name)
     try:
         return globals()[formatter_name](settings)
     except RuntimeError:
@@ -113,6 +114,9 @@ class Formatter(object):
         except BrokenPipeError:
             pass
 
+    def verify(self):
+        return True
+
 
 class NullFormatter(Formatter):
 
@@ -292,25 +296,32 @@ class SummaryFormatter(Formatter):
 
             for s in sorted(r.series_names):
                 self.write((" %-" + str(txtlen) + "s : ") % s)
-                d = [i for i in r.series(s) if i is not None]
+                try:
+                    d = [i[1] for i in r.raw_series(s) if i[1] is not None]
+                except KeyError:
+                    d = None
 
-                median = mean = None
+                if not d:
+                    d = [i for i in r.series(s) if i is not None]
 
-                if s in m and 'MEAN_VALUE' in m[s]:
-                    mean = m[s]['MEAN_VALUE']
-                elif not d:
+                md = m.get(s, {})
+
+                units = (md.get('UNITS') or
+                         self.settings.DATA_SETS.get(s, {}).get('units', ''))
+
+                mean = md.get('MEAN_VALUE')
+                median = md.get('RTT_MEDIAN') if units == 'ms' else None
+
+                if not mean and not d:
                     self.write("No data.\n")
                     continue
 
-                units = self.settings.DATA_SETS[s]['units'] if s in \
-                    self.settings.DATA_SETS else ''
-
                 if d and self.np is not None:
-                    mean = self.np.mean(d) if not mean else mean
-                    median = self.np.median(d)
+                    mean = mean or self.np.mean(d)
+                    median = median or self.np.median(d)
                 elif d:
-                    mean = sum(d) / len(d) if not mean else mean
-                    median = sorted(d)[len(d) // 2]
+                    mean = mean or sum(d) / len(d)
+                    median = median or sorted(d)[len(d) // 2]
 
                 if mean and units == 'bytes':
                     factor, units = format_bytes(max(mean, median))
@@ -348,19 +359,26 @@ class PlotFormatter(Formatter):
 
         self.figure = None
         self.plotter = None
-        self.init_plots()
 
-    def init_plots(self):
-        if self.figure is None:
-            self.plotter = self.plotters.new(self.settings)
-            self.plotter.init()
-            self.figure = self.plotter.figure
-        else:
+    def init_plots(self, results=None):
+        if self.figure is not None:
             self.figure.clear()
             self.plotter.disable_cleanup = True
             self.plotter.disconnect_callbacks()
-            self.plotter = self.plotters.new(self.settings, figure=self.figure)
-            self.plotter.init()
+
+        if self.settings.SUBPLOT_COMBINE:
+            self.plotter = self.plotters.new(self.settings,
+                                             plotter=self.plotters.get_plotter(
+                                                 "subplot_combine"),
+                                             figure=self.figure,
+                                             results=results)
+        else:
+            self.plotter = self.plotters.new(self.settings,
+                                             figure=self.figure,
+                                             results=results)
+
+        self.figure = self.plotter.figure
+        self.plotter.init()
 
     @property
     def disable_cleanup(self):
@@ -374,18 +392,13 @@ class PlotFormatter(Formatter):
         if not results[0]:
             return
 
-        if self.settings.SUBPLOT_COMBINE:
-            self.plotter.disable_cleanup = True
-            self.plotter.disconnect_callbacks()
-            self.figure.clear()
-            self.plotter = self.plotters.new(self.settings,
-                                             plotter=self.plotters.get_plotter(
-                                                 "subplot_combine"),
-                                             figure=self.figure)
-            self.plotter.init()
+        self.init_plots(results)
         self.plotter.plot(results)
         self.plotter.save(results)
 
+    def verify(self):
+        return self.plotter.verify()
+
 
 class MetadataFormatter(Formatter):
 
