@@ -28,6 +28,21 @@ from . import test_parsers
 from . import test_plotters
 from . import test_tests
 
+class TestResult(unittest.TextTestResult):
+    def _exc_info_to_string(self, err, test):
+        exctype, value, tb = err
+
+        if hasattr(value, 'orig_tb'):
+            return str(value) + ":\n\n" + value.orig_tb
+
+        return super(TestResult, self)._exc_info_to_string(err, test)
+
+class TestRunner(unittest.TextTestRunner):
+    def __init__(self, *args, **kwargs):
+        kwargs['resultclass'] = TestResult
+        kwargs['verbosity'] = 2
+        super(TestRunner, self).__init__(*args, **kwargs)
+
 test_suite = unittest.TestSuite([test_util.test_suite,
                                  test_metadata.test_suite,
                                  test_parsers.test_suite,
